From e290abd6286836c3db8ee3d11d5ac4bbc8081e01 Mon Sep 17 00:00:00 2001
Message-Id: <e290abd6286836c3db8ee3d11d5ac4bbc8081e01.1553676165.git.jan.steffens@gmail.com>
From: "Jan Alexander Steffens (heftig)" <jan.steffens@gmail.com>
Date: Wed, 27 Mar 2019 00:38:02 +0100
Subject: [PATCH] wayland/output: Report unscaled size even in logical layout
 mode

GTK divides the size by the scale factor to get the logical output size.
Presumably other clients do so, as well. Without this patch, mutter
would report the nonsensical 960x540 for a 3840x2160@2 output.

Fixes part of https://bugzilla.mozilla.org/show_bug.cgi?id=1534089
---
 src/wayland/meta-wayland-outputs.c | 33 +++++++++++++++++++-----------
 1 file changed, 21 insertions(+), 12 deletions(-)

diff --git a/src/wayland/meta-wayland-outputs.c b/src/wayland/meta-wayland-outputs.c
index 7695d86af..712a143a8 100644
--- a/src/wayland/meta-wayland-outputs.c
+++ b/src/wayland/meta-wayland-outputs.c
@@ -181,24 +181,36 @@ send_output_events (struct wl_resource *resource,
   MetaLogicalMonitor *old_logical_monitor;
   guint old_mode_flags;
   gint old_scale;
+  gint scale;
+  MetaRectangle old_rect;
+  MetaRectangle rect;
   float old_refresh_rate;
   float refresh_rate;
 
   old_logical_monitor = wayland_output->logical_monitor;
   old_mode_flags = wayland_output->mode_flags;
   old_scale = wayland_output->scale;
+  old_rect = old_logical_monitor->rect;
   old_refresh_rate = wayland_output->refresh_rate;
 
+  scale = calculate_wayland_output_scale (logical_monitor);
+  rect = logical_monitor->rect;
   monitor = pick_main_monitor (logical_monitor);
-
   current_mode = meta_monitor_get_current_mode (monitor);
   refresh_rate = meta_monitor_mode_get_refresh_rate (current_mode);
 
+  if (meta_is_stage_views_scaled ()) {
+    old_rect.width *= old_scale;
+    old_rect.height *= old_scale;
+    rect.width *= scale;
+    rect.height *= scale;
+  }
+
   gboolean need_done = FALSE;
 
   if (need_all_events ||
-      old_logical_monitor->rect.x != logical_monitor->rect.x ||
-      old_logical_monitor->rect.y != logical_monitor->rect.y ||
+      old_rect.x != rect.x ||
+      old_rect.y != rect.y ||
       is_different_rotation (old_logical_monitor, logical_monitor))
     {
       int width_mm, height_mm;
@@ -229,40 +241,37 @@ send_output_events (struct wl_resource *resource,
       transform = WL_OUTPUT_TRANSFORM_NORMAL;
 
       wl_output_send_geometry (resource,
-                               logical_monitor->rect.x,
-                               logical_monitor->rect.y,
+                               rect.x,
+                               rect.y,
                                width_mm,
                                height_mm,
                                subpixel_order,
                                vendor,
                                product,
                                transform);
       need_done = TRUE;
     }
 
   preferred_mode = meta_monitor_get_preferred_mode (monitor);
   if (current_mode == preferred_mode)
     mode_flags |= WL_OUTPUT_MODE_PREFERRED;
 
   if (need_all_events ||
-      old_logical_monitor->rect.width != logical_monitor->rect.width ||
-      old_logical_monitor->rect.height != logical_monitor->rect.height ||
+      old_rect.width != rect.width ||
+      old_rect.height != rect.height ||
       old_refresh_rate != refresh_rate ||
       old_mode_flags != mode_flags)
     {
       wl_output_send_mode (resource,
                            mode_flags,
-                           logical_monitor->rect.width,
-                           logical_monitor->rect.height,
+                           rect.width,
+                           rect.height,
                            (int32_t) (refresh_rate * 1000));
       need_done = TRUE;
     }
 
   if (version >= WL_OUTPUT_SCALE_SINCE_VERSION)
     {
-      int scale;
-
-      scale = calculate_wayland_output_scale (logical_monitor);
       if (need_all_events ||
           old_scale != scale)
         {
-- 
2.21.0

